import matplotlib
matplotlib.use('Cairo')
import matplotlib.pyplot as plt
from numpy import *
import os

plt.rcParams['figure.figsize'] = 10,7

xlim = 25000
xlimmodulo = 250

def onlymodulo(l):
  res = []
  for i,e in zip(xrange(xlim),l):
    if i % xlimmodulo == 0:
      res.append(e)
  return res

def plotdatafile(filepath,fm,lb,normalize=None):
  x, y, l, h = [], [], [], []
  for line in open(filepath).readlines():
    tokens = line[0:-1].split(' ')
    x.append(int(tokens[0]))
    y.append(float(tokens[1]))
    l.append(float(tokens[2]))
    h.append(float(tokens[3]))
  x, y, l, h = onlymodulo(x), onlymodulo(y), onlymodulo(l), onlymodulo(h)
  xx, yy, ll, hh = array(x), array(y), array(l), array(h)
  ll = yy - ll
  hh = hh - yy
  if normalize != None:
    xx = xx * normalize
    plt.plot(xx,yy,fm,label=lb)
  else:
    plt.gca().errorbar(xx,yy,yerr=(ll,hh),fmt=fm,label=lb)

def plotsrfile(filepath,lb,maxruns=None):
  lines = open(filepath).readlines()
  nbruns = maxruns if maxruns != None else len(lines)
  for i in xrange(nbruns):
    x, y = [], []
    tokens = lines[i][0:-1].split()
    for j in xrange(len(tokens)):
      x.append(j)
      y.append(float(tokens[j]))
    x, y = onlymodulo(x), onlymodulo(y)
    xx, yy = array(x), array(y)
    plt.plot(xx,yy,"k-",label=(i==0 and lb or None))

plotformats = ['c-','r-','g-','m-']

# STATS

plt.figure(1)
plt.subplot(211)
plt.gca().set_title('distance to closest optimum (natural gradient)')
for i,f in zip(xrange(0,100),filter(lambda x: x.startswith('ng'), sorted(os.listdir('stats')))):
  lr = f[f.rfind('-')+3:]
  plotdatafile('stats/'+f,plotformats[i],'$\delta t = '+lr+'$')
plt.gca().set_xlim(0,xlim)
plt.gca().set_ylim(0,20)
plt.gca().legend()
plt.grid()
plt.xlabel('gradient steps')

plt.subplot(212)
plt.gca().set_title('distance to closest optimum (vanilla gradient)')
for i,f in zip(xrange(0,100),filter(lambda x: x.startswith('vg'), sorted(os.listdir('stats')))):
  lr = f[f.rfind('-')+3:]
  plotdatafile('stats/'+f,plotformats[i],'$\delta t = '+lr+'$')
plt.gca().set_xlim(0,xlim)
plt.gca().set_ylim(0,20)
plt.gca().legend()
plt.grid()
plt.xlabel('gradient steps')
plt.subplots_adjust(left  = 0.125, right = 0.9, bottom = 0.1, top = 0.9, wspace = 0.2, hspace = 0.3)
plt.savefig('stats.eps',bbox_inches='tight')

# M-STATS

plt.figure(2)
plt.subplot(211)
plt.gca().set_title('distance to second optimum (natural gradient)')
for i,f in zip(xrange(0,100),filter(lambda x: x.startswith('ng'), sorted(os.listdir('mstats')))):
  lr = f[f.rfind('-')+3:]
  plotdatafile('mstats/'+f,plotformats[i],'$\delta t = '+lr+'$')
plt.gca().set_xlim(0,xlim)
plt.gca().set_ylim(0,40)
plt.gca().legend()
plt.grid()
plt.xlabel('gradient steps')

plt.subplot(212)
plt.gca().set_title('distance to second optimum (vanilla gradient)')
for i,f in zip(xrange(0,100),filter(lambda x: x.startswith('vg'), sorted(os.listdir('mstats')))):
  lr = f[f.rfind('-')+3:]
  plotdatafile('mstats/'+f,plotformats[i],'$\delta t = '+lr+'$')
plt.gca().set_xlim(0,xlim)
plt.gca().set_ylim(0,40)
plt.gca().legend(loc='best')
plt.grid()
plt.xlabel('gradient steps')
plt.subplots_adjust(left  = 0.125, right = 0.9, bottom = 0.1, top = 0.9, wspace = 0.2, hspace = 0.3)
plt.savefig('mstats.eps',bbox_inches='tight')

# LR-indep STATS/MSTATS

# H-STATS

plt.figure(3)
plt.subplot(211)
plt.gca().set_title('h-statistics (natural gradient)')
for i,f in zip(xrange(0,100),filter(lambda x: x.startswith('ng'), sorted(os.listdir('hstats')))):
  lr = f[f.rfind('-')+3:]
  plotdatafile('hstats/'+f,plotformats[i],'$\delta t = '+lr+'$')
plt.gca().set_xlim(0,xlim)
plt.gca().set_ylim(0,1)
plt.gca().legend()
plt.grid()
plt.xlabel('gradient steps')

plt.subplot(212)
plt.gca().set_title('h-statistics (vanilla gradient)')
for i,f in zip(xrange(0,100),filter(lambda x: x.startswith('vg'), sorted(os.listdir('hstats')))):
  lr = f[f.rfind('-')+3:]
  plotdatafile('hstats/'+f,plotformats[i],'$\delta t = '+lr+'$')
plt.gca().set_xlim(0,xlim)
plt.gca().set_ylim(0,1)
plt.gca().legend()
plt.grid()
plt.xlabel('gradient steps')
plt.subplots_adjust(left  = 0.125, right = 0.9, bottom = 0.1, top = 0.9, wspace = 0.2, hspace = 0.3)
plt.savefig('hstats.eps',bbox_inches='tight')

# SINGLE RUNS
figurenum = 3
srfigs = []

for i,fn,fv in zip(xrange(0,100),filter(lambda x: x.startswith('ng'), sorted(os.listdir('sr1'))),filter(lambda x: x.startswith('vg'), sorted(os.listdir('sr1')))):
  lrn = fn[fn.rfind('-')+3:]
  lrv = fv[fv.rfind('-')+3:]
  figurenum = figurenum + 1
  plt.figure(figurenum)

  plt.subplot(221)
  plt.gca().set_title('dist. to closest opt. (nat. grad.)')
  plotsrfile('sr1/ng-lr'+lrn,'$\delta t = '+lrn+'$',maxruns=10)
  plt.gca().set_xlim(0,xlim)
  plt.gca().set_ylim(0,30)
  plt.gca().legend()
  plt.gca().set_xticks([0,5000,10000,15000,20000])
  plt.grid()
  plt.xlabel('gradient steps')

  plt.subplot(222)
  plt.gca().set_title('dist. to closest opt. (van. grad.)')
  plotsrfile('sr1/vg-lr'+lrv,'$\delta t = '+lrv+'$',maxruns=10)
  plt.gca().set_xlim(0,xlim)
  plt.gca().set_ylim(0,30)
  plt.gca().legend()
  plt.gca().set_xticks([0,5000,10000,15000,20000])
  plt.grid()
  plt.xlabel('gradient steps')

  plt.subplot(223)
  plt.gca().set_title('dist. to second opt. (nat. grad.)')
  plotsrfile('sr2/ng-lr'+lrn,'$\delta t = '+lrn+'$',maxruns=10)
  plt.gca().set_xlim(0,xlim)
  plt.gca().set_ylim(0,30)
  plt.gca().legend()
  plt.gca().set_xticks([0,5000,10000,15000,20000])
  plt.grid()
  plt.xlabel('gradient steps')

  plt.subplot(224)
  plt.gca().set_title('dist. to second opt. (van. grad.)')
  plotsrfile('sr2/vg-lr'+lrv,'$\delta t = '+lrv+'$',maxruns=10)
  plt.gca().set_xlim(0,xlim)
  plt.gca().set_ylim(0,30)
  plt.gca().legend()
  plt.gca().set_xticks([0,5000,10000,15000,20000])
  plt.grid()
  plt.xlabel('gradient steps')
  
  figname = 'sr-'+str(figurenum-3)+'.eps'
  plt.subplots_adjust(left  = 0.125, right = 0.9, bottom = 0.1, top = 0.9, wspace = 0.2, hspace = 0.3)
  plt.savefig(figname,bbox_inches='tight')
  srfigs.append(figname)


os.system("convert stats.eps mstats.eps hstats.eps "+(" ".join(srfigs))+" results-31.pdf")

plt.show()
