#include "experiments.h"

#include <algorithm>
#include "Random.h"

/// Optimum is at 0.
double sphere (clap::Matrix<double> x) {
  return x.l1Norm();
}

/// Stand in function always returning 0.
double zero(clap::Matrix<double> x){
  return 0;
}

/// The function onemax with an isolated optimum.
double bigjump (clap::Matrix<double> x, int t) {
  double m = x.l1Norm();
  if( m == x.size() ) return x.size();
  else if ( m > x.size() - t ) return 0;
  else return m;
}

/// four optima : two local and two global + isolation.
double fourpeaks (clap::Matrix<double> x, int t) {
  // number of leading ones
  int o = 0;
  while ( x(o)==1 ) o += 1;
  // number of trailing zeros
  int z = 0;
  while ( x(x.size()-z-1) == 0 ) z += 1;
  // reward
  if ( o > t && z > t ) return std::max(o,z) + x.size();
  else return std::max(o,z);
}

/// base functor for the equal products function. give the dimension
/// as argument to create a usable functor;
struct equalproductsFunctor {
  equalproductsFunctor(int dim){
    clap::Random<double> rng;
    dimension = dim;
    a = new double[dimension];
    for(int i = 0; i < dimension; ++i){
      a[i] = rng.uniform(0,4);
    }
  }
  ~equalproductsFunctor(){
    delete[] a;
  }
  double operator()(clap::Matrix<double> x){
    ASSERT( x.size() == dimension );
    double p1 = 1;
    double p2 = 1;
    for(int i = 0; i < dimension; ++i){
      if( x(i) == 0 ) p1 = p1 * a[i];
      else p2 = p2 * a[i];
    }
    return abs(p1-p2);
  }
  int dimension;
  double* a;
};

/// 0 if x is in 0*1*
double zerosThenOnes (clap::Matrix<double> x) {
  int i = 0;
  while ( x (i) == 0 && i < x.size() ) i += 1;
    while ( i < x.size() ) if ( x (i) != 1 ) return 1000; else i += 1;
  return 0;
}

